@@ -19,6 +19,8 @@
 # Author: Aamir Khan <syst3m.w0rm@gmail.com>
 #
 
+# pylint: skip-file
+
 class Email(object):
     """ Email class containing the information needed to store and
     display email threads.
@@ -39,6 +41,7 @@ class Email(object):
         self.liked = 0
         self.author = ''
         self.avatar = None
+        self.age = '6 days'
 
 class Author(object):
     """ Author class containing the information needed to get the top
