@@ -19,13 +19,19 @@
 # Author: Aurelien Bompard <abompard@fedoraproject.org>
 #
 
+from __future__ import absolute_import, unicode_literals
 
 import datetime
+from functools import wraps
 
+from django.http import Http404
 from django.utils.timezone import utc
+from django.utils.decorators import available_attrs
+from django.shortcuts import render
 
-from hyperkitty.models import ThreadCategory, LastView
+from hyperkitty.models import ThreadCategory, MailingList, Profile
 from hyperkitty.views.forms import CategoryForm
+from hyperkitty.lib.cache import cache
 
 
 FLASH_MESSAGES = {
@@ -35,7 +41,7 @@ FLASH_MESSAGES = {
 }
 
 
-def get_months(store, list_name):
+def get_months(mlist):
     """ Return a dictionnary of years, months for which there are
     potentially archives available for a given list (based on the
     oldest post on the list).
@@ -43,7 +49,11 @@ def get_months(store, list_name):
     :arg list_name, name of the mailing list in which this email
     should be searched.
     """
-    date_first = store.get_start_date(list_name)
+    date_first = cache.get_or_set(
+        "MailingList:%s:first_date" % mlist.name,
+        lambda: mlist.emails.order_by("date"
+            ).values_list("date", flat=True).first(),
+        None)
     if not date_first:
         return {}
     archives = {}
@@ -63,7 +73,8 @@ def get_display_dates(year, month, day):
         start_day = 1
     else:
         start_day = int(day)
-    begin_date = datetime.datetime(int(year), int(month), start_day)
+    begin_date = datetime.datetime(
+        int(year), int(month), start_day, tzinfo=utc)
 
     if day is None:
         end_date = begin_date + datetime.timedelta(days=32)
@@ -79,7 +90,7 @@ def daterange(start_date, end_date):
         yield start_date + datetime.timedelta(n)
 
 
-def get_category_widget(request=None, current_category=None):
+def get_category_widget(request=None, current_category=None, categories=()):
     """
     Returns the category form and the applicable category object (or None if no
     category is set for this thread).
@@ -89,9 +100,6 @@ def get_category_widget(request=None, current_category=None):
     if request is not provided or None, don't return the category form, return
     None instead.
     """
-    categories = [ (c.name, c.name.upper())
-                   for c in ThreadCategory.objects.all() ] \
-                 + [("", "no category")]
 
     if request:
         if request.method == "POST":
@@ -115,51 +123,47 @@ def get_category_widget(request=None, current_category=None):
     return category, category_form
 
 
-def is_thread_unread(request, mlist_name, thread):
-    """Returns True or False if the thread is unread or not."""
-    unread = False
-    if request.user.is_authenticated():
-        try:
-            last_view_obj = LastView.objects.get(
-                    list_address=mlist_name,
-                    threadid=thread.thread_id,
-                    user=request.user)
-        except LastView.DoesNotExist:
-            unread = True
-        else:
-            if thread.date_active.replace(tzinfo=utc) \
-                    > last_view_obj.view_date:
-                unread = True
-    return unread
-
-
-def get_recent_list_activity(store, mlist):
-    """Return the number of emails posted in the last 30 days"""
-    begin_date, end_date = mlist.get_recent_dates()
-    days = daterange(begin_date, end_date)
-
-    # Use get_messages and not get_threads to count the emails, because
-    # recently active threads include messages from before the start date
-    emails_in_month = store.get_message_dates(
-            list_name=mlist.name, start=begin_date, end=end_date)
-    # graph
-    emails_per_date = {}
-    # populate with all days before adding data.
-    for day in days:
-        emails_per_date[day.strftime("%Y-%m-%d")] = 0
-    # now count the emails
-    for email_date in emails_in_month:
-        date_str = email_date.strftime("%Y-%m-%d")
-        if date_str not in emails_per_date:
-            continue # outside the range
-        emails_per_date[date_str] += 1
-    # return the proper format for the javascript chart function
-    return [ {"date": d, "count": emails_per_date[d]}
-             for d in sorted(emails_per_date) ]
-
-
 def show_mlist(mlist, request):
     def get_domain(host):
         return ".".join(host.split(".")[-2:])
     return (get_domain(mlist.name.partition("@")[2])
             == get_domain(request.get_host()))
+
+
+# View decorator: check that the list is authorized
+def check_mlist_private(func):
+    @wraps(func, assigned=available_attrs(func))
+    def inner(request, *args, **kwargs):
+        if "mlist_fqdn" in kwargs:
+            mlist_fqdn = kwargs["mlist_fqdn"]
+        else:
+            mlist_fqdn = args[0]
+        try:
+            mlist = MailingList.objects.get(name=mlist_fqdn)
+        except MailingList.DoesNotExist:
+            raise Http404("No archived mailing-list by that name.")
+        if not is_mlist_authorized(request, mlist):
+            return render(request, "hyperkitty/errors/private.html", {
+                            "mlist": mlist,
+                          }, status=403)
+        return func(request, *args, **kwargs)
+    return inner
+
+
+def is_mlist_authorized(request, mlist):
+    if not mlist.is_private:
+        return True
+    if not request.user.is_authenticated():
+        return False
+    # Private list and logged-in user: check subscriptions
+    try:
+        profile = Profile.objects.get(user_id=request.user.id)
+    except Profile.DoesNotExist:
+        # Create the profile if it does not exist. There's a signal receiver
+        # that creates it for new users, but HyperKitty may be added to an
+        # existing Django project with existing users.
+        profile = Profile.objects.create(user=request.user)
+    if mlist.name in profile.get_subscriptions():
+        return True
+    else:
+        return False
