@@ -26,6 +26,17 @@ function setup_index(url_template) {
         document.location.href = $(this).find("a.list-name").attr("href");
     });
 
+    // Helper to load the graph
+    function show_ajax_chart(listelem) {
+        if (!listelem.is(":visible")) {
+            return;
+        }
+        var listname = $.trim(listelem.find(".list-address").text());
+        url_template = url_template.replace(/@/, "%40"); // Django 1.5 compatibility, it did not escape the url tag
+        var url = url_template.replace(/PLACEHOLDER%40PLACEHOLDER/, listname);
+        ajax_chart(listelem.find("div.chart"), url, {height: 30});
+    }
+
     // Filter
     function filter_lists() {
         var hide_by_class = {};
@@ -42,46 +53,43 @@ function setup_index(url_template) {
             var list_name = $.trim($(this).find("a.list-name").text());
             var list_addr = $.trim($(this).find("a.list-address").text());
             var must_hide = false;
+            // name filter
             if (filter && list_name.indexOf(filter) === -1
                        && list_addr.indexOf(filter) === -1) {
                 must_hide = true;
             }
+            // class filter
             for (cls in hide_by_class) {
                 if ($(this).hasClass(cls) && hide_by_class[cls]) {
                     must_hide = true;
                 }
             }
+            // now apply the filters
             if (must_hide) {
                 $(this).hide();
             } else {
                 $(this).show();
+                show_ajax_chart($(this));
             }
         });
     }
     $(".hide-switches input").click(filter_lists);
+    var _filter_timeout = null;
     $(".filter-lists input").change(function() {
+        clearTimeout(_filter_timeout)
         // reset status according to the "hide" checkboxes
-        filter_lists();
+        window.setTimeout(filter_lists, 500);
     }).keyup(function() {
         // fire the above change event after every letter
         $(this).change();
     }).focus();
+    filter_lists(); // Filter on page load
 
-    // Initials
-    $(".initials").animate({ right: 0 }, {duration: 600});
-    // Override the scrolling because we have a fixed header
-    $(".initials a").click(function (e) {
-        e.preventDefault();
-        var target = $("a[name="+$(this).attr("href").substring(1)+"]");
-        $(window).scrollTop(target.offset().top - 70);
-    });
+    // Back to top link
+    setup_back_to_top_link(220); // set offset to 220 for link to appear
 
-    // Update list graphs
-    $(".all-lists table.lists tr.list").each(function() {
-        var listelem = $(this);
-        var listname = $.trim(listelem.find(".list-address").text());
-        url_template = url_template.replace(/@/, "%40"); // Django 1.5 compatibility, it did not escape the url tag
-        var url = url_template.replace(/PLACEHOLDER%40PLACEHOLDER/, listname);
-        ajax_chart(listelem.find("div.chart"), url, {height: 30});
+    // Update list graphs for visible lists
+    $(".all-lists table.lists tr.list:visible").each(function() {
+        show_ajax_chart($(this));
     });
 }
