@@ -19,29 +19,29 @@
 # Author: Aurelien Bompard <abompard@fedoraproject.org>
 #
 
+from __future__ import absolute_import, unicode_literals, print_function
+
 import datetime
 import re
+from collections import OrderedDict
 
 import json
 from dateutil.tz import tzoffset
 from django import template
-from django.utils.datastructures import SortedDict
 from django.utils.html import conditional_escape
 from django.utils.safestring import mark_safe
 from django.utils.timezone import utc
 
+import hyperkitty.lib.posting
+from hyperkitty.lib.utils import stripped_subject
+
 register = template.Library()
 
 
-@register.filter
-def trimString(str):
-    return re.sub('\s+', ' ', str)
-
-
 @register.filter(name='sort')
 def listsort(value):
     if isinstance(value, dict):
-        new_dict = SortedDict()
+        new_dict = OrderedDict()
         key_list = value.keys()
         key_list.sort()
         key_list.reverse()
@@ -65,24 +65,6 @@ def to_date(month, year):
     return datetime.date(year, month, 1)
 
 
-@register.filter
-def strip_page(value):
-    if not value:
-        return value
-    if value.endswith('/') and value[-3] == '/':
-        end_with_number = False
-        try:
-            if int(value[-2]) in range(0,10):
-                end_with_number = True
-            if end_with_number:
-                output = value.rsplit('/', 2)
-        except ValueError:
-            output = value.rsplit('/', 1)
-    else:
-        output = value.rsplit('/', 1)
-    return output[0]
-
-
 # From http://djangosnippets.org/snippets/1259/
 @register.filter
 def truncatesmart(value, limit=80):
@@ -127,21 +109,15 @@ def escapeemail(text):
 
 
 @register.filter()
-def get_date(email_or_thread):
+def date_with_senders_timezone(email):
     """
-    Rebuild the date of an email or a thread taking the timezone into account.
+    Rebuild the date of an email taking the sender's timezone into account.
     """
-    if hasattr(email_or_thread, 'date'):
-        tz = tzoffset(None, email_or_thread.timezone * 60)
-        date_obj = email_or_thread.date.replace(tzinfo=tz)
-    elif hasattr(email_or_thread, 'date_active'):
-        date_obj = email_or_thread.date_active.replace(tzinfo=utc)
-    else:
-        raise ValueError(email_or_thread)
-    return date_obj
+    tz = tzoffset(None, email.timezone * 60)
+    return email.date.astimezone(tz)
 
 
-SNIPPED_RE = re.compile("^(\s*&gt;).*$", re.M)
+SNIPPED_RE = re.compile(r"^(\s*&gt;).*$", re.M)
 @register.filter(needs_autoescape=True)
 def snip_quoted(content, quotemsg="...", autoescape=None):
     """Snip quoted text in messages"""
@@ -167,13 +143,48 @@ def snip_quoted(content, quotemsg="...", autoescape=None):
                 current_quote_orig = []
         #lastline = line
     for quote_orig, quote in quoted:
-        replaced = ('<div class="quoted-switch"><a href="#">%s</a></div>' % quotemsg
+        replaced = ('<div class="quoted-switch"><a style="font-weight:normal" href="#">%s</a></div>' % quotemsg
                    +'<div class="quoted-text">'
                    +"\n".join(quote)
                    +' </div>')
         content = content.replace("\n".join(quote_orig), replaced)
     return mark_safe(content)
 
+SNIPPED_BEGIN_PGP = re.compile("^.*(BEGIN PGP SIGNATURE).*$", re.M)
+SNIPPED_END_PGP = re.compile("^.*(END PGP SIGNATURE).*$", re.M)
+@register.filter(needs_autoescape=True)
+def snip_pgp(content, quotemsg="...PGP SIGNATURE...", autoescape=None):
+    """Snip pgp signature in messages"""
+    if autoescape:
+        content = conditional_escape(content)
+    quoted = []
+    current_quote = []
+    current_quote_orig = []
+    pgp_signature = False
+    for line in content.split("\n"):
+        match_start = SNIPPED_BEGIN_PGP.match(line)
+        match_end = SNIPPED_END_PGP.match(line)
+        if match_end is not None or match_start is not None or pgp_signature:
+            current_quote_orig.append(line)
+            current_quote.append(line)
+        # start of pgp signature
+        if match_start is not None:
+            pgp_signature = True
+        # end of pgp signature
+        elif match_end is not None:
+            pgp_signature = False
+            if current_quote_orig:
+                current_quote_orig.append("")
+                quoted.append( (current_quote_orig[:], current_quote[:]) )
+                current_quote = []
+                current_quote_orig = []
+    for quote_orig, quote in quoted:
+        replaced = ('<div class="quoted-switch"><a href="#" class="pgp">%s</a></div>' % quotemsg
+                   +'<div class="quoted-text">'
+                   +"\n".join(quote)
+                   +' </div>')
+        content = content.replace("\n".join(quote_orig), replaced)
+    return mark_safe(content)
 
 @register.filter()
 def multiply(num1, num2):
@@ -216,10 +227,33 @@ def to_json(value):
     return json.dumps(value)
 
 
+@register.filter
+def get_item(dictionary, key):
+    return dictionary.get(key)
+
+
 @register.filter(is_safe=True)
-def num_comments(value):
+def num_comments(thread):
     """Returns the number of comments in a thread"""
     try:
-        return len(value) - 1
+        return thread.emails_count - 1
     except (ValueError, TypeError):
         return ''
+
+
+@register.filter
+def reply_subject(value):
+    return hyperkitty.lib.posting.reply_subject(value)
+
+
+@register.filter(name="strip_subject")
+def strip_subject(subject, mlist):
+    return stripped_subject(mlist, subject)
+
+@register.filter
+def is_unread_by(thread, user):
+    return thread.is_unread_by(user)
+
+@register.filter
+def sort_by_name(p_list):
+    return sorted(p_list, key=lambda p: p.name.lower())
