@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (C) 1998-2012 by the Free Software Foundation, Inc.
+# Copyright (C) 2014-2015 by the Free Software Foundation, Inc.
 #
 # This file is part of HyperKitty.
 #
@@ -20,28 +20,29 @@
 # Author: Aurelien Bompard <abompard@fedoraproject.org>
 #
 
+from __future__ import absolute_import, unicode_literals
+
 import datetime
 import re
 import json
-from collections import namedtuple
 
 from django.http import HttpResponse, Http404
 from django.template import RequestContext, loader
-from django.shortcuts import render, redirect
+from django.shortcuts import render, redirect, get_object_or_404
 from django.core.urlresolvers import reverse
 from django.core.exceptions import SuspiciousOperation
 from django.utils.timezone import utc
 import robot_detection
+from haystack.query import SearchQuerySet
 
-from hyperkitty.models import Tag, Favorite, LastView
+from hyperkitty.models import Tag, Tagging, Favorite, LastView, Thread, MailingList, ThreadCategory
 from hyperkitty.views.forms import AddTagForm, ReplyForm
-from hyperkitty.lib import get_store, stripped_subject
+from hyperkitty.lib.utils import stripped_subject
 from hyperkitty.lib.view_helpers import (get_months, get_category_widget,
-        FLASH_MESSAGES)
-from hyperkitty.lib.mailman import check_mlist_private
+        FLASH_MESSAGES, check_mlist_private)
 
 
-def _get_thread_replies(request, thread, offset=1, limit=None):
+def _get_thread_replies(request, thread, limit, offset=1):
     '''
     Get and sort the replies for a thread.
     By default, offset = 1 to skip the original message.
@@ -49,88 +50,93 @@ def _get_thread_replies(request, thread, offset=1, limit=None):
     if not thread:
         raise Http404
 
-    if "sort" in request.GET and request.GET["sort"] == "date":
-        sort_mode = "date"
-        emails = thread.emails
-    else:
-        sort_mode = "thread"
-        emails = thread.emails_by_reply
+    sort_mode = request.GET.get("sort", "thread")
+    if sort_mode not in ("date", "thread"):
+        raise SuspiciousOperation
+    if sort_mode == "thread":
+        sort_mode = "thread_order"
 
-    # XXX: Storm-specific
-    emails = emails.find()
-    emails.config(offset=offset, limit=limit)
-
-    emails = list(emails)
+    mlist = thread.mailinglist
+    initial_subject = stripped_subject(mlist, thread.starting_email.subject)
+    emails = list(thread.emails.order_by(sort_mode)[offset:offset+limit])
     for email in emails:
         # Extract all the votes for this message
-        email.myvote = email.get_vote_by_user_id(request.session.get("user_id"))
-        if sort_mode == "thread":
+        if request.user.is_authenticated():
+            email.myvote = email.votes.filter(user=request.user).first()
+        else:
+            email.myvote = None
+        # Threading position
+        if sort_mode == "thread_order":
             email.level = email.thread_depth - 1 # replies start ragged left
             if email.level > 5:
                 email.level = 5
-
+            elif email.level < 0:
+                email.level = 0
+        # Subject change
+        subject = stripped_subject(mlist, email.subject)
+        if subject.lower().startswith("re:"):
+            subject = subject[3:].strip()
+        if subject.strip() == initial_subject.strip():
+            email.changed_subject = False
+        else:
+            email.changed_subject = subject
     return emails
 
 
 @check_mlist_private
 def thread_index(request, mlist_fqdn, threadid, month=None, year=None):
     ''' Displays all the email for a given thread identifier '''
-    store = get_store(request)
-    thread = store.get_thread(mlist_fqdn, threadid)
-    if not thread:
-        raise Http404
-    prev_thread, next_thread = store.get_thread_neighbors(mlist_fqdn, threadid)
+    # pylint: disable=unused-argument
+    mlist = get_object_or_404(MailingList, name=mlist_fqdn)
+    thread = get_object_or_404(Thread, mailinglist=mlist, thread_id=threadid)
+    starting_email = thread.starting_email
 
     sort_mode = request.GET.get("sort", "thread")
-    thread.starting_email.myvote = thread.starting_email.get_vote_by_user_id(
-            request.session.get("user_id"))
+    if request.user.is_authenticated():
+        starting_email.myvote = starting_email.votes.filter(
+            user=request.user).first()
+    else:
+        starting_email.myvote = None
 
     # Tags
     tag_form = AddTagForm()
-    try:
-        tags = Tag.objects.filter(threadid=threadid, list_address=mlist_fqdn)
-    except Tag.DoesNotExist:
-        tags = []
 
     # Favorites
     fav_action = "add"
-    if request.user.is_authenticated():
-        try:
-            Favorite.objects.get(list_address=mlist_fqdn, threadid=threadid,
-                                 user=request.user)
-        except Favorite.DoesNotExist:
-            pass
-        else:
-            fav_action = "rm"
+    if request.user.is_authenticated() and Favorite.objects.filter(
+            thread=thread, user=request.user).exists():
+        fav_action = "rm"
 
     # Category
-    category, category_form = get_category_widget(request, thread.category)
+    categories = [ (c.name, c.name.upper())
+                   for c in ThreadCategory.objects.all() ] \
+                 + [("", "no category")]
+    category, category_form = get_category_widget(
+        request, thread.category, categories)
 
     # Extract relative dates
     today = datetime.date.today()
-    days_old = today - thread.starting_email.date.date()
-    days_inactive = today - thread.last_email.date.date()
+    days_old = today - starting_email.date.date()
+    days_inactive = today - thread.date_active.date()
 
-    mlist = store.get_list(mlist_fqdn)
-    subject = stripped_subject(mlist, thread.starting_email.subject)
+    subject = stripped_subject(mlist, starting_email.subject)
 
     # Last view
     last_view = None
     if request.user.is_authenticated():
         last_view_obj, created = LastView.objects.get_or_create(
-                list_address=mlist_fqdn, threadid=threadid, user=request.user)
+                thread=thread, user=request.user)
         if not created:
             last_view = last_view_obj.view_date
             last_view_obj.save() # update timestamp
     # get the number of unread messages
     if last_view is None:
         if request.user.is_authenticated():
-            unread_count = len(thread)
+            unread_count = thread.emails_count
         else:
             unread_count = 0
     else:
-        # XXX: Storm-specific
-        unread_count = thread.replies_after(last_view).count()
+        unread_count = thread.emails.filter(date__gt=last_view).count()
 
     # Flash messages
     flash_messages = []
@@ -142,53 +148,49 @@ def thread_index(request, mlist_fqdn, threadid, month=None, year=None):
 
     # TODO: eventually move to a middleware ?
     # http://djangosnippets.org/snippets/1865/
-    is_bot = True
     user_agent = request.META.get('HTTP_USER_AGENT', None)
     if user_agent:
         is_bot = robot_detection.is_robot(user_agent)
+    else:
+        is_bot = True
 
     context = {
         'mlist': mlist,
-        'threadid': threadid,
+        'thread': thread,
+        'starting_email': starting_email,
         'subject': subject,
-        'tags': tags,
         'addtag_form': tag_form,
         'month': thread.date_active,
-        'first_mail': thread.starting_email,
-        'neighbors': (prev_thread, next_thread),
-        'months_list': get_months(store, mlist.name),
+        'months_list': get_months(mlist),
         'days_inactive': days_inactive.days,
         'days_old': days_old.days,
         'sort_mode': sort_mode,
         'fav_action': fav_action,
         'reply_form': ReplyForm(),
         'is_bot': is_bot,
-        'num_comments': len(thread) - 1,
-        'participants': thread.participants,
+        'num_comments': thread.emails_count - 1,
         'last_view': last_view,
         'unread_count': unread_count,
         'category_form': category_form,
         'category': category,
         'flash_messages': flash_messages,
     }
-    context["participants"].sort(key=lambda p: p.name.lower())
 
     if is_bot:
         # Don't rely on AJAX to load the replies
         # The limit is a safety measure, don't let a bot kill the DB
         context["replies"] = _get_thread_replies(request, thread, limit=1000)
 
-    return render(request, "thread.html", context)
+    return render(request, "hyperkitty/thread.html", context)
 
 
 @check_mlist_private
 def replies(request, mlist_fqdn, threadid):
     """Get JSON encoded lists with the replies and the participants"""
-    chunk_size = 5
+    chunk_size = 6 # must be an even number, or the even/odd cycle will be broken
     offset = int(request.GET.get("offset", "1"))
-    store = get_store(request)
-    thread = store.get_thread(mlist_fqdn, threadid)
-    mlist = store.get_list(mlist_fqdn)
+    thread = get_object_or_404(Thread,
+        mailinglist__name=mlist_fqdn, thread_id=threadid)
     # Last view
     last_view = request.GET.get("last_view")
     if last_view:
@@ -197,15 +199,14 @@ def replies(request, mlist_fqdn, threadid):
         except ValueError:
             last_view = None
     context = {
-        'mlist': mlist,
-        'threadid': threadid,
+        'threadid': thread,
         'reply_form': ReplyForm(),
         'last_view': last_view,
     }
     context["replies"] = _get_thread_replies(request, thread, offset=offset,
                                              limit=chunk_size)
 
-    replies_tpl = loader.get_template('ajax/replies.html')
+    replies_tpl = loader.get_template('hyperkitty/ajax/replies.html')
     replies_html = replies_tpl.render(RequestContext(request, context))
     response = {"replies_html": replies_html,
                 "more_pending": False,
@@ -224,6 +225,8 @@ def tags(request, mlist_fqdn, threadid):
     if not request.user.is_authenticated():
         return HttpResponse('You must be logged in to add a tag',
                             content_type="text/plain", status=403)
+    thread = get_object_or_404(Thread,
+        mailinglist__name=mlist_fqdn, thread_id=threadid)
 
     if request.method != 'POST':
         raise SuspiciousOperation
@@ -241,31 +244,28 @@ def tags(request, mlist_fqdn, threadid):
         raise SuspiciousOperation
     tagnames = [ t.strip() for t in re.findall(r"[\w'_ -]+", tagname) ]
     for tagname in tagnames:
-        try:
-            tag = Tag.objects.get(threadid=threadid, list_address=mlist_fqdn,
-                                  tag=tagname)
-        except Tag.DoesNotExist:
-            if action == "add":
-                tag = Tag(list_address=mlist_fqdn, threadid=threadid,
-                          tag=tagname, user=request.user)
-                tag.save()
-            elif action == "rm":
+        if action == "add":
+            tag = Tag.objects.get_or_create(name=tagname)[0]
+            Tagging.objects.get_or_create(
+                tag=tag, thread=thread, user=request.user)
+        elif action == "rm":
+            try:
+                Tagging.objects.get(tag__name=tagname, thread=thread,
+                                    user=request.user).delete()
+            except Tagging.DoesNotExist:
                 raise Http404("No such tag: %s" % tagname)
-        else:
-            if action == "rm":
-                tag.delete()
+            # cleanup
+            if not Tagging.objects.filter(tag__name=tagname).exists():
+                Tag.objects.filter(name=tagname).delete()
 
     # Now refresh the tag list
-    tags = Tag.objects.filter(threadid=threadid, list_address=mlist_fqdn)
-    FakeMList = namedtuple("MailingList", ["name"])
-    tpl = loader.get_template('threads/tags.html')
+    tpl = loader.get_template('hyperkitty/threads/tags.html')
     html = tpl.render(RequestContext(request, {
-            "tags": tags,
-            "mlist": FakeMList(name=mlist_fqdn),
-            "threadid": threadid,
+            "thread": thread,
             }))
 
-    response = {"tags": [ t.tag for t in tags ], "html": html}
+    response = {"tags": [ t.name for t in thread.tags.distinct() ],
+                "html": html}
     return HttpResponse(json.dumps(response),
                         content_type='application/javascript')
 
@@ -273,15 +273,20 @@ def tags(request, mlist_fqdn, threadid):
 def suggest_tags(request, mlist_fqdn, threadid):
     term = request.GET.get("term")
     current_tags = Tag.objects.filter(
-            list_address=mlist_fqdn, threadid=threadid
-            ).values_list("tag", flat=True)
+            threads__mailing_list__name=mlist_fqdn,
+            threads__thread_id=threadid
+        ).values_list("name", flat=True)
     if term:
-        tags = Tag.objects.filter(list_address=mlist_fqdn, tag__istartswith=term)
+        tags_db = Tag.objects.filter(
+            threads__mailing_list__name=mlist_fqdn,
+            name__istartswith=term)
     else:
-        tags = Tag.objects.all()
-    tags = tags.exclude(tag__in=current_tags).values_list("tag", flat=True)
-    tags = [ t.encode("utf8") for t in tags ]
-    return HttpResponse(json.dumps(tags), content_type='application/javascript')
+        tags_db = Tag.objects.all()
+    tag_names = [ t.encode("utf-8") for t in
+             tags_db.exclude(name__in=current_tags
+                ).values_list("name", flat=True)[:20] ]
+    return HttpResponse(json.dumps(tag_names),
+                        content_type='application/javascript')
 
 
 @check_mlist_private
@@ -290,24 +295,15 @@ def favorite(request, mlist_fqdn, threadid):
     if not request.user.is_authenticated():
         return HttpResponse('You must be logged in to have favorites',
                             content_type="text/plain", status=403)
-
     if request.method != 'POST':
         raise SuspiciousOperation
 
-    props = dict(list_address=mlist_fqdn, threadid=threadid, user=request.user)
+    thread = get_object_or_404(Thread,
+        mailinglist__name=mlist_fqdn, thread_id=threadid)
     if request.POST["action"] == "add":
-        try:
-            fav = Favorite.objects.get(**props)
-        except Favorite.DoesNotExist:
-            fav = Favorite(**props)
-        fav.save()
+        Favorite.objects.get_or_create(thread=thread, user=request.user)
     elif request.POST["action"] == "rm":
-        try:
-            fav = Favorite.objects.get(**props)
-        except Favorite.DoesNotExist:
-            pass
-        else:
-            fav.delete()
+        Favorite.objects.filter(thread=thread, user=request.user).delete()
     else:
         raise SuspiciousOperation
     return HttpResponse("success", content_type='text/plain')
@@ -322,25 +318,23 @@ def set_category(request, mlist_fqdn, threadid):
     if request.method != 'POST':
         raise SuspiciousOperation
 
-    store = get_store(request)
+    thread = get_object_or_404(Thread,
+        mailinglist__name=mlist_fqdn, thread_id=threadid)
     category, category_form = get_category_widget(request)
-    thread = store.get_thread(mlist_fqdn, threadid)
     if not category and thread.category:
         thread.category = None
-        store.commit()
+        thread.save()
     elif category and category.name != thread.category:
         thread.category = category.name
-        store.commit()
+        thread.save()
 
     # Now refresh the category widget
-    FakeMList = namedtuple("MailingList", ["name"])
     context = {
             "category_form": category_form,
-            "mlist": FakeMList(name=mlist_fqdn),
-            "threadid": threadid,
+            "thread": thread,
             "category": category,
             }
-    return render(request, "threads/category.html", context)
+    return render(request, "hyperkitty/threads/category.html", context)
 
 
 @check_mlist_private
@@ -348,58 +342,57 @@ def reattach(request, mlist_fqdn, threadid):
     if not request.user.is_staff:
         return HttpResponse('You must be a staff member to reattach a thread',
                             content_type="text/plain", status=403)
-    flash_messages = []
-    store = get_store(request)
-    mlist = store.get_list(mlist_fqdn)
-    thread = store.get_thread(mlist_fqdn, threadid)
+    mlist = get_object_or_404(MailingList, name=mlist_fqdn)
+    thread = get_object_or_404(Thread, mailinglist=mlist, thread_id=threadid)
+    context = {
+        'mlist': mlist,
+        'thread': thread,
+        'months_list': get_months(mlist),
+        'flash_messages': [],
+    }
+    template_name = "hyperkitty/reattach.html"
 
     if request.method == 'POST':
         parent_tid = request.POST.get("parent")
         if not parent_tid:
             parent_tid = request.POST.get("parent-manual")
-        if not parent_tid or not re.match("\w{32}", parent_tid):
-            flash_messages.append({"type": "warning",
-                                   "msg": "Invalid thread id, it should look "
-                                          "like OUAASTM6GS4E5TEATD6R2VWMULG44NKJ."})
-        elif parent_tid == threadid:
-            flash_messages.append({"type": "warning",
-                                   "msg": "Can't re-attach a thread to "
-                                          "itself, check your thread ID."})
-        else:
-            new_thread = store.get_thread(mlist_fqdn, parent_tid)
-            if new_thread is None:
-                flash_messages.append({"type": "warning",
-                                       "msg": "Unknown thread, check your "
-                                              "thread ID."})
-            elif thread.starting_email.date <= new_thread.starting_email.date:
-                flash_messages.append({"type": "error",
-                                       "msg": "Can't attach an older thread "
-                                              "to a newer thread."})
-            else:
-                for msg in thread.emails:
-                    store.attach_to_thread(msg, new_thread)
-                store.delete_thread(mlist_fqdn, threadid)
-                return redirect(reverse(
-                        'thread', kwargs={
-                            "mlist_fqdn": mlist_fqdn,
-                            'threadid': parent_tid,
-                        })+"?msg=attached-ok")
-
-
-    context = {
-        'mlist' : mlist,
-        'thread': thread,
-        'months_list': get_months(store, mlist.name),
-        'flash_messages': flash_messages,
-    }
-    return render(request, "reattach.html", context)
+        if not parent_tid or not re.match(r"\w{32}", parent_tid):
+            context["flash_messages"].append(
+                {"type": "warning",
+                 "msg": "Invalid thread id, it should look "
+                        "like OUAASTM6GS4E5TEATD6R2VWMULG44NKJ."})
+            return render(request, template_name, context)
+        if parent_tid == threadid:
+            context["flash_messages"].append(
+                {"type": "warning",
+                 "msg": "Can't re-attach a thread to "
+                        "itself, check your thread ID."})
+            return render(request, template_name, context)
+        try:
+            parent = Thread.objects.get(
+                mailinglist=thread.mailinglist, thread_id=parent_tid)
+        except Thread.DoesNotExist:
+            context["flash_messages"].append(
+                {"type": "warning",
+                 "msg": "Unknown thread, check your thread ID."})
+            return render(request, template_name, context)
+        try:
+            thread.starting_email.set_parent(parent.starting_email)
+        except ValueError as e:
+            context["flash_messages"].append({"type": "error", "msg": str(e)})
+            return render(request, template_name, context)
+        return redirect(reverse(
+                'hk_thread', kwargs={
+                    "mlist_fqdn": mlist_fqdn,
+                    'threadid': parent_tid,
+                })+"?msg=attached-ok")
+    return render(request, template_name, context)
 
 
 @check_mlist_private
 def reattach_suggest(request, mlist_fqdn, threadid):
-    store = get_store(request)
-    mlist = store.get_list(mlist_fqdn)
-    thread = store.get_thread(mlist_fqdn, threadid)
+    mlist = get_object_or_404(MailingList, name=mlist_fqdn)
+    thread = get_object_or_404(Thread, mailinglist=mlist, thread_id=threadid)
 
     default_search_query = stripped_subject(
         mlist, thread.subject).lower().replace("re:", "")
@@ -407,18 +400,17 @@ def reattach_suggest(request, mlist_fqdn, threadid):
     if not search_query:
         search_query = default_search_query
     search_query = search_query.strip()
-    if store.search_index:
-        search_result = store.search(search_query, mlist_fqdn, 1, 50)
-        messages = search_result["results"]
-    else:
-        messages = []
+    messages = SearchQuerySet().filter(
+        mailinglist__exact=mlist_fqdn, content=search_query)[:50]
     suggested_threads = []
     for msg in messages:
-        if msg.thread not in suggested_threads and msg.thread_id != threadid:
-            suggested_threads.append(msg.thread)
+        sugg_thread = msg.object.thread
+        if sugg_thread not in suggested_threads \
+            and sugg_thread.thread_id != threadid:
+            suggested_threads.append(sugg_thread)
 
     context = {
         'mlist' : mlist,
         'suggested_threads': suggested_threads[:10],
     }
-    return render(request, "ajax/reattach_suggest.html", context)
+    return render(request, "hyperkitty/ajax/reattach_suggest.html", context)
